#include "ClientLink.h"

#define MAX_NODE 128		// for hard coded char array

ClientLink::ClientLink(int p, int port, int dataport, int debug,
			World *w, MasterMPI *m)
{
	if (debug>2)
		DEBUG = 1;
	else
		DEBUG = 0;

	world = w;
	mysocket = new Server(port, dataport);
	num_p = p;
	mpi = m;
}

void ClientLink::recvParams()
{
	int i;

	/* client wants us to send us sim parameters */

	double param_pack[PARAM_SIZE_CLIENT];

	if (DEBUG)
		printf("Waiting to recv parameters from client.\n");

	mysocket->recv_doubles(param_pack, PARAM_SIZE_CLIENT);
	if (DEBUG)
	{
		printf("I recv'd the following parameters:\n");
		for (i=0; i<PARAM_SIZE_CLIENT; i++)
			printf("%0.5f, ", param_pack[i]);
		printf("\n");
	}

	// now put the recv'd packet into our world's param object

	world->param.setParamsClient(param_pack);
}

void ClientLink::recvWorld()
{
	int i;

	/* client wants us to send us shapes */
	
	if (DEBUG)
		printf("Waiting to receive shapes from client.\n");

	int A[1];
	int num_shapes;

	mysocket->recv_ints(A,1);
	num_shapes = A[0];
	
	// we recv one packet with all the shapes and the canvas
	// width and height on the end of it

	double all_shape[MAX_ATTRS * num_shapes + 2];
	double shape[MAX_ATTRS];

	// instantiate the world shape array
	world->theGrid->setNumShapes(num_shapes);

	Point p, end;

	int count = 0;

	// recv all the shapes at one time
	mysocket->recv_doubles(all_shape,MAX_ATTRS * num_shapes + 2);

	for (i=0; i<num_shapes; i++)
	{

		// separate out the attributes of a particular shape

		for (int j=0; j<MAX_ATTRS; j++)
			shape[j] = all_shape[i*MAX_ATTRS + j];

		// tell the world about the attributes of this new shape

		world->theGrid->addShape(shape);
	}

	world->setContainerWidth(all_shape[MAX_ATTRS * num_shapes]);
	world->setContainerHeight(all_shape[MAX_ATTRS * num_shapes + 1]);

	if (DEBUG)
	{
		cout << "I recv'd the following shape attributes:" << endl;
		for (i=0; i<num_shapes*MAX_ATTRS; i++)
			cout << shape[i] << " ";
 		cout << endl;
	
		cout << "Container width = " <<
			world->getContainerWidth() << " height = " <<
				world->getContainerHeight() << endl;
	}
}

void ClientLink::connectInit()
{
	// wait for a socket connection and then receive the 
	// parameters, shapes and network parameters from the client

	mysocket->connect();

	// now we do a performance stats update of all our nodes
	double results[num_p];
	char hostname[MAX_NODE][80];

        // clear the things out for sure
        // kdv 5/28
        for (int i = 0; i < MAX_NODE; i++)
               for (int j = 0; j < 80; j++)
                       hostname[i][j] = '\0';

	mpi->runStats(results, hostname);
	sendStats(results, hostname);

	recvAll();
}

void ClientLink::recvAll()
{
	// get a sequenced update of the params, world and network 

	getCommand();			// we know what it is
	recvParams();

	getCommand();			// we know what it is
	recvWorld();
}

int ClientLink::getCommand()
{
	int A[1];

	if (DEBUG)
		printf("Server waiting for command on socket...\n");

	mysocket->recv_ints(A,1);

	return A[0];
}

void ClientLink::bye()
{
	if (DEBUG)
		printf("Server closing socket...\n");

	mysocket->closesocket();
}

void ClientLink::sendStats(double results[], char hostname[MAX_NODE][80])
{
	// send the stats down the pipe

	// tell the client how many nodes we have
	int A[1];
	A[0] = num_p;
	mysocket->send_ints(A, 1);

	for (int i=0; i<num_p; i++)
	{
		mysocket->send_string(hostname[i]);
	}

	mysocket->send_doubles(results, num_p);
}

// Send the client all the information it will need to know about this
// frame in the simulation (particle data, shape data, min/max/avg)

void ClientLink::sendClientFrame()
{
        // tell the client how many floats to expect
        int A[1];
        float *buffer;
        A[0] = world->theGrid->getFrame(&buffer);

        mysocket->send_ints(A, 1);

        // now send them off

        mysocket->send_floats(buffer, A[0]);
}

// Tell the client that the simulation is finished
// (it will be expecting this to be another frame)
void ClientLink::simDone()
{
	int A[1];
	A[0] = -1;
	mysocket->send_ints(A, 1);
}

