//***********************************************************
//
// A virtual Shape object. Overloads "Shape". This is actually a
// 2D Cylinder (circle).
//
// Written by: Brian Schlatter
// Last updated: 6/12/1999
//
//********************************************************** 


#ifndef CYLINDER_HPP
#define CYLINDER_HPP

#include <iostream.h>
#include <stdlib.h>
#include <assert.h>
#include "Point.h"
#include <math.h>
#include "Shape.h"

// This is used in the update method for keeping variables
// consisitent when one is changed.
enum VARTYPE {RADIUS, CENTER, WIDTH, HEIGHT, ULC};

class Cylinder :  public Shape {
	public:
		Cylinder();
		Cylinder(Point center_XY, double r);
		Cylinder(const Cylinder & C);
		Cylinder & operator = (const Cylinder & C);
		~Cylinder() {}

		//Accessor methods
		void setCenterPt(Point c) { center = c; update(CENTER); } 
		void setCenterXY(double x, double y) 
			{ center.setXY(x, y); update(CENTER);               } 
		void setRadius(double r)  { radius = r; update(RADIUS); }
		void setCornerPoint(Point xy) 
			{ lowerLeftCorner = xy; update(ULC);                }
		void setCornerXYPos(double x_c, double y_c) 
			{ lowerLeftCorner.setXY(x_c, y_c); update(ULC);     }
		void setWidth(double w)   { width = w; update(WIDTH);   }
		void setHeight(double h)  { height = h; update(HEIGHT); }


		double getRadius() const  { return radius;              }
		Point getCenter()  const  { return center;              }

		//Miscellaneous methods.
		double area() const;
		void update(VARTYPE);
		int isInside(double , double );

		double calcRatioOfDistances(Point , Point );
		friend ostream & operator << (ostream & , const Cylinder & );
		friend ofstream & operator << (ofstream & , const Cylinder & );	
		
	protected:
		double radius;
		Point center;

};

#endif



