//**********************************************************
//* This class is a linked list data structure with O(1) 
//* access, addition, and removal time. It does this by
//* assuming that one wants to go through the entire list
//* at a time; thus, it only needs to maintain a "current" 
//* pointer.
//*
//* Written by: Brian Schlatter
//* Last modified: 5/3/99
//***********************************************************


#ifndef FASTLINKEDLIST_HPP
#define FASTLINKEDLIST_HPP 

#include <stddef.h>
#include <iostream.h>
#include <assert.h>

// We store the index of a smoothparticle that resides in
//an array.
typedef int itemType;

struct listNode;
typedef listNode* ptrType;

class FastLinkedList {
	public:
		FastLinkedList();
		~FastLinkedList();

		//Get the size of the list.
		int length()  { return Size; }

		//check to see if it is empty
		int isEmpty() {return !Size; };

		//Methods to move pointers along the list.
		void nextCur(); 
		void nextOther(); 
		void firstCur();
		void firstOther();

		//Add NewItem to the front of the list.
		void push(itemType NewItem);

		//Remove the item that **** is pointing to.
		void deleteCur();
		void deleteOther();
		
		//Get the item that pointer **** is pointing to
		itemType getCur();
		itemType getOther();

		// delete the whole linked list
		void deleteAll();

		// print out what is in the list
		void print();
		
	private:
		int Size;
		ptrType Head;
		ptrType Tail;
		ptrType Cur;
		ptrType Other;
};

#endif
