
// This canvas appears on the main application screen when the user selects
// the network view.  Displays a bar graph of the relative perform of
// the nodes in the network.

import java.awt.*;
import com.sun.java.swing.*;

class NetworkCanvas extends JPanel {
    private Font font1, font2;
    private sphclient myparent;
    
    public NetworkCanvas(Frame parent)
    {
        // constructor
    	font1 = new Font("Monospaced", Font.PLAIN, 10);
        font2 = new Font("Monospaced", Font.PLAIN, 12);
        
        myparent = (sphclient) parent;        
	}

    public void paint (Graphics g)
    {
        // make sure things are blank to start with
        g.setColor(getBackground());
        g.fillRect(0,0,4000,4000);
        
        g.setFont(font2);
        g.setColor(Color.blue);
        
        if (myparent.network.connected)
        {
            // put client, server and socket info at the top
            
            g.drawString("Client: " + myparent.network.clientname, 5, 10);
            g.drawString("Server: " + myparent.network.server, 5, 24);
            g.drawString("Talking on socket port " + myparent.network.port, 5, 38);
            if (myparent.network.reverse==1)
                g.drawString("Reversed byte order requested.", 5, 52);
            else
                g.drawString("Normal byte order requested.", 5, 52);                            
                
            // we want to put up a graph of the system performance
            // stats recv'd from the server

            // first lets put up the names of the machines
            int i;
            double max;
            int max_name_len = 0;
            int max_bar;

            final int graphat = 60;     // y position the graph starts at
            
            g.setFont(font1);
            for (i=0; i<myparent.network.totalnodes; i++)
            {
                g.drawString(myparent.network.nodenames.elementAt(i).toString(), 5, i*12 + graphat + 8);
                
                if (g.getFontMetrics().stringWidth(myparent.network.nodenames.elementAt(i).toString())>max_name_len)
                    max_name_len = g.getFontMetrics().stringWidth(myparent.network.nodenames.elementAt(i).toString());
            }

            max_name_len = max_name_len + 10;  // stand off a bit
            
            // now find the maximum performance value

            max = -9999.0;

            for (i=0; i<myparent.network.totalnodes; i++)
            {
                if (((Double) myparent.network.nodeflops.elementAt(i)).doubleValue()>max)
                    max = ((Double) myparent.network.nodeflops.elementAt(i)).doubleValue();
            }

            // normalize everybody to a maximum of 1.0
            // then multiply by the max width we want the bar

            max_bar = this.getSize().width - max_name_len - 100;

            double[] mysize = new double[myparent.network.totalnodes];
            
            for (i=0; i<myparent.network.totalnodes; i++)
            {
                mysize[i] = ((Double) myparent.network.nodeflops.elementAt(i)).doubleValue() / max * max_bar;
            }

            // draw the bars
            g.setColor(Color.red);
            for (i=0; i<myparent.network.totalnodes; i++)
            {
                g.fillRect(max_name_len, i*12 + graphat, (int) mysize[i], 7);
            }

            // put in the decimal megaflop readings
            g.setColor(Color.blue);
            for (i=0; i<myparent.network.totalnodes; i++)
            {
                Double d;
                d = (Double) myparent.network.nodeflops.elementAt(i);

                if ((d.toString()).length() < 8)
                    g.drawString((d.toString()).substring(0,(d.toString()).length())+ " mflops", this.getSize().width-90, i*12 + graphat + 8);
                else
                    g.drawString((d.toString()).substring(0,7) + " mflops", this.getSize().width-90, i*12 + graphat + 8);
            }
        }
        else
        {
            // we are not connected
            
            g.drawString("We are not connected.",10,20);
        }
    }

    public void update()
    {
        repaint();
    }
    
    private int canvas_x(double world_x)
    {
        // convert the passed x world coordinate to a canvas x
        
        Dimension d = getSize();        
        return (int) ((world_x / myparent.world.width) * d.width);
    }

    private int scale_x(double world_x)
    {
        // convert the passed x width to a canvas width
        
        Dimension d = getSize();        
        return (int) ((world_x / myparent.world.width) * d.width);
    }

    private int canvas_y(double world_y)
    {
        // convert the passed y world coordinate to a canvas y
        // note: y coordinate is flipped, java's canvas 0,0 is
        // upper-left, but our simulation assumes lower-left
        
        Dimension d = getSize();        
        return (int) (d.height - ((world_y / myparent.world.height) * d.height));
    }    

    private int scale_y(double world_y)
    {
        Dimension d = getSize();        
        return (int) ((world_y / myparent.world.height) * d.height);
    }    

}