#include "Parameter.h"

//******************************************************************
// Default constructor.
//******************************************************************
Parameter::Parameter(): numPts(0), velocityScale(0), initVel(0, 0), 
initMass(0), initDensity(0), initPressure(0), initPtSpacing(0), 
h(0), c(0), machNum(0), deltaT(0), outputEvery(0), betaMax(0), 
numSteps(0), firstOutput(0), nu(0), bodyFX(0), bodyFY(0), 
densityVariation(0), lengthScale(0) 
{}

//******************************************************************
// Copy constructor
//******************************************************************
Parameter::Parameter(const Parameter & P)
{
	numPts		        = P.numPts;
	velocityScale	            = P.velocityScale;
	initVel	            = P.initVel;
	initMass	        = P.initMass;
	initDensity	        = P.initDensity;
	initPressure        = P.initPressure;
	initPtSpacing       = P.initPtSpacing;
	h	                = P.h;
	c	                = P.c;
	machNum             = P.machNum;
	firstOutput			= P.firstOutput;
	numSteps			= P.numSteps;
	outputEvery			= P.outputEvery;
	deltaT				= P.deltaT;
	betaMax				= P.betaMax;
	nu					= P.nu;
	bodyFX				= P.bodyFX;
	bodyFY				= P.bodyFY;
	densityVariation    = P.densityVariation;
	lengthScale  		 = P.lengthScale;
}	


//******************************************************************
// overloaded "=" operator
//******************************************************************
Parameter & Parameter::operator = (const Parameter & P)
{
	// A check to make sure we're not trying to copy ourself to
	// ourself. If this did happen, then we would delete all
	// information before we had a chance to copy it.
	if (this == &P)
		return *this;
	
	numPts		        	= P.numPts;
	velocityScale	            	= P.velocityScale;
	initVel	            	= P.initVel;
	initMass	        	= P.initMass;
	initDensity	        	= P.initDensity;
	initPressure        	= P.initPressure;
	initPtSpacing       	= P.initPtSpacing;
	h	                	= P.h;
	c	                	= P.c;
	machNum             	= P.machNum;
	firstOutput				= P.firstOutput;
	numSteps				= P.numSteps;	
	outputEvery				= P.outputEvery;
	deltaT					= P.deltaT;
	betaMax					= P.betaMax;
	nu						= P.nu;
	bodyFX				= P.bodyFX;
	bodyFY				= P.bodyFY;
	densityVariation    = P.densityVariation;
	lengthScale	  		 = P.lengthScale;

	return (*this);
}



//******************************************************************
//  overloaded ostream operator (standard output)
//******************************************************************
ostream & operator << ( ostream & str, const Parameter & P)
{
	str << "numPts = " << P.numPts <<endl;
	str << "velocityScale = "  << P.velocityScale<<endl;
	str << "initVel = " << P.initVel <<endl;
	str << "initMass = " << P.initMass <<endl;
	str << "initDensity = " << P.initDensity <<endl;
	str << "initPressure = " << P.initPressure <<endl;
	str << "initPtSpacing = "  << P.initPtSpacing <<endl;
	str << "h = " << P.h <<endl;
	str << "c = " << P.c <<endl;
	str << "machNum = " << P.machNum  <<endl;
	str << "outputEvery = " << P.outputEvery << endl;
	str << "firstOutput = " << P.firstOutput << endl;
	str << "numSteps = " << P.numSteps << endl;
	str << "deltaT = " << P.deltaT << endl;
	str << "betaMax = " << P.betaMax << endl;
	str << "nu = " << P.nu << endl;
	str << "bodyFX =" << P.bodyFX << endl;
	str << "bodyFY =" << P.bodyFY << endl;
	str << "densityVariation =" << P.densityVariation << endl;
	str << "lengthScale =" << P.lengthScale << endl;	
	
	return str;
}	

//******************************************************************
// Stores all values in Parameter into array d[].
//******************************************************************
void Parameter::getParams(double d[PARAM_SIZE])
{
        d[0] = (double) numPts;
        d[1] = velocityScale;
        d[2] = initVel.getX();
        d[3] = initVel.getY();
        d[4] = initMass;
        d[5] = initDensity;
        d[6] = initPressure;
        d[7] = initPtSpacing;
        d[8] = h;
        d[9] = c;
        d[10] = machNum;
        d[11] = betaMax;
	d[12] = nu;
	d[13] = (double) outputEvery;
	d[14] = (double) firstOutput;
	d[15] = (double) numSteps;
	d[16] = deltaT;
	d[17] = bodyFX;
	d[18] = bodyFY;
 	d[19] = densityVariation;
	d[20] = lengthScale;
}

//******************************************************************
// Sets all values in Parameter to the values in array d[]
//******************************************************************
void Parameter::setParams(double d[PARAM_SIZE])
{
        numPts = (int) d[0];
        velocityScale = d[1];
        initVel.setX(d[2]);
        initVel.setY(d[3]);
        initMass = d[4];
        initDensity = d[5];
        initPressure = d[6];
        initPtSpacing = d[7];
        h = d[8];
        c = d[9];
        machNum = d[10];
        betaMax = d[11];
	nu = d[12];
	outputEvery = (int) d[13];
	firstOutput = (int) d[14];
	numSteps = (int) d[15];
	deltaT = d[16];
	bodyFX = d[17];			
	bodyFY = d[18];		
 	densityVariation = d[19];
	lengthScale = d[20];
}


//******************************************************************
// Sets all values in Parameter to the values in array d[]
// This version works with a packet from the client.
// The client only has the parameters that the user can set, none
// of the calculated parameters.
//******************************************************************
void Parameter::setParamsClient(double d[PARAM_SIZE_CLIENT])
{
        initVel.setX(d[0]);
        initVel.setY(d[1]);
        velocityScale 		= 	d[2];
        machNum 		= 	d[3];
        numPts 			= (int) d[4];
        initDensity 		= 	d[5];
	deltaT			= 	d[6];
	nu 			= 	d[7];
	outputEvery 		= (int) d[8];
	firstOutput 		= (int) d[9];
	numSteps 		= (int) d[10];
	bodyFX			= 	d[11];	
	bodyFY			= 	d[12];	
 	densityVariation 	= 	d[13];
	lengthScale 		= 	d[14];
}



