//***********************************************************
//
// A base class for the shapes that we will put in our flow.
//
// Written by: Brian Schlatter
// Last updated: 6/12/1999
//
//********************************************************** 
#ifndef SHAPE_HPP
#define SHAPE_HPP

#define MAX_ATTRS 5  // This is the number of variables we use 
                     // to describe a shape; type is included.

#include <iostream.h>
#include <stdlib.h>
#include <assert.h>
#include "msgtags.h"
#include "Point.h"

#define BOX 1
#define CYLINDER 2

class Shape {
	public:
		// Constructor & destructor methods
		Shape();
		Shape(int name);
		Shape(Point xy, double w, double h, int name);
		Shape(const Shape & S); 
		~Shape(){}
		
			
		//Accessor methods
		void setCornerPoint(Point xy) { lowerLeftCorner = xy;    }
		void setCornerXYPos(double x_c, double y_c) 
				                { lowerLeftCorner.setXY(x_c, y_c);    }
		void setWidth(double w) 	  { width = w;                     }
		void setHeight(double h)	  { height = h;                    }
		Point getCornerPoint() const  { return lowerLeftCorner;        }
		double getCornerX()    const  { return lowerLeftCorner.getX(); }
		double getCornerY()    const  { return lowerLeftCorner.getY(); }
		double getWidth()      const  { return width;                  }
		double getHeight()     const  { return height;                 }
		int getType()          const  { return type;                   }
		char * getType(int)    const; 
		
		//Miscellaneous methods.
		virtual double area() const { return 0.0; } 
		virtual int isInside(double x, double y) { return 0; }
		virtual double calcRatioOfDistances(Point, Point) {return 0.0; }

		Shape & operator = (const Shape & S); 
		friend ostream & operator << (ostream & str, const Shape & s);
		friend ofstream & operator << (ofstream & fStr, const Shape & s);
		
	protected:
		Point lowerLeftCorner;
		double width, height;
		int type;
};
		
#endif
