#ifndef SLAVEGRID_HPP
#define SLAVEGRID_HPP

#include <stdlib.h>
#include <assert.h>

#include "Parameter.h"
#include "Cell.h"
#include "Point.h"
#include "SmoothParticle.h"
#include "FastLinkedList.h"
#include "msgtags.h"
#include "Timer.h"
#include "Grid.h"
#include <iomanip.h>
#include <iostream.h>

class SlaveGrid : public Grid {

	public:
		SlaveGrid();
		SlaveGrid(Parameter );
		SlaveGrid(const SlaveGrid & G);
		~SlaveGrid();
		
		void calcPressure();
		void calcSPHEqns();
		void calcCorrectDeltaT();
		
		void predictParticles();
		void setRangeOfRows(int, int);

		// These are used just by the slaves's grid
		void deleteBufferRows();
		void deleteNorthBound();
		void deleteSouthBound();

		// These are used when we need to migrate particles to another processor
		int getNorthBound(int **);
		int getSouthBound(int **);
		int getNorthFringe(int **);
		int getSouthFringe(int **);


		// Used to send updates to master
		int getBody(int **);

		void setSouthMost(int a)  {southMost = a;}
		void setNorthMost(int a)  {northMost = a;}
		int getSouthMost()        {return southMost;}
		int getNorthMost()        {return northMost;}

		void updateParticles();
		void updateParticlesFirstLoop();
		void updateGrid();

		SlaveGrid & operator = (const SlaveGrid & );
		friend ostream & operator << ( ostream & , const SlaveGrid & );
		friend ofstream & operator << ( ofstream & , const SlaveGrid & );

		void switchBound();

	private:
		FastLinkedList northBoundPts;
		FastLinkedList southBoundPts;
		int northMost, southMost;
		void calcFreeParticleInteraction(int, int, int);
		void calcBndParticleInteraction(int, int, int);
		void calcVaB(Point & , SmoothParticle & , SmoothParticle & );
		void deleteRow(int cellIndexJ);
		
};

#endif
