//***********************************************************
//
// This class contains all variables and methods that allow a 
// Smooth Particle to interact with other smoothparticles 
// and its environment.
//
// Written by: Brian Schlatter
// Last updated: 6/12/1999
//
//********************************************************** 
#ifndef SMOOTHPARTICLE_HPP
#define SMOOTHPARTICLE_HPP

#include <math.h>
#include "Particle.h"
#include "Parameter.h"
#include "msgtags.h"

class SmoothParticle : public Particle {
	public:
		SmoothParticle();
		SmoothParticle(Point p, Point v, double m, double d, double pr, 
			           double sph_len, double speedSnd);
		SmoothParticle(const SmoothParticle & SP);
		~SmoothParticle() {}

		void setPressure(double p)            { pressure = p;             }
		void setDensity(double den)           { density = den;            }
		void setSmoothingLength(double len)   { h = len;                  }
		void setSpeedSound(double spSnd)      { c = spSnd;                }
		void setChangeVelocity(Point ChangeV) { ChangeVelocity = ChangeV; }
		void setChangeDensity(double cd)      { ChangeDensity = cd;       }
		void setMinDensity(double m)		  { minDensity = m;  			}
		void setMaxDensity(double m)		  { maxDensity = m;  			}
		void setShapeID(int id)		          { shapeID = id;   }		
		void setNumNeighbors(int n)	      { numNeighbors = n;   }		

		double getPressure()        const { return pressure;            }
		double getSmoothingLength() const { return h;                   }
		double getDensity()         const { return density;             }
		double getSpeedSound()      const { return c;                   }
		double getChangeDensity()   const { return ChangeDensity;         }
		Point getChangeVelocity()   const { return ChangeVelocity;        }
		double getChangeVelocityX() const { return ChangeVelocity.getX(); }
		double getChangeVelocityY() const { return ChangeVelocity.getY(); }
		int getShapeID() 	     	const { return shapeID; }
		double getMinDensity()	    const { return minDensity;      }
		double getMaxDensity()	    const { return maxDensity;      }
		int getNumNeighbors()	    const { return numNeighbors;    }

		void zeroSPHVars();
		void calcChangeDensity(const SmoothParticle & , Point );
		void calcChangeVelocity(const SmoothParticle &, Point , const Parameter & );
		void calcPressure(const Parameter &);
		void addForceX(const Parameter &);
		void addForceY(const Parameter &);
		void setMinMaxDensity();
		void updateNumNeighbors(const SmoothParticle &);

		void getPacket(double D[SMOOTH_SIZE]);
		void getPacketClient(float D[SMOOTH_SIZE_CLIENT]);
		void setPacket(double D[SMOOTH_SIZE]);
								
		SmoothParticle & operator = (const SmoothParticle & SP);
		friend ostream & operator << (ostream & str, const SmoothParticle & sp);
		friend ofstream & operator << (ofstream & fStr, const SmoothParticle & sp);

		int ID;
		
	protected:
		double pressure;
		double density;
		double h;
		double c;
		double ChangeDensity;
		Point ChangeVelocity;
		int shapeID;		
		int numNeighbors;	

		double minDensity;
		double maxDensity;

		double kernel(const SmoothParticle & );
		double artificialViscosity(const SmoothParticle &, const Parameter &);
		double gradientKernelX(const SmoothParticle & );
		double gradientKernelY(const SmoothParticle & );

	private:	
		
		double pressureTerm(const SmoothParticle & );

};


#endif



