// SPH simulation using MPI

// This is the main program for both the master and slave processes
// It handles the main loop logic for the simulation, details are
// hidden behind the master and slave objects

// version 1.00

#include "mpi.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>

#include "Master.h"
#include "Slave.h"

#include "mpe.h"

/*
debugging level:     0 - none
		     1 - main only
		     2 - main, master, slave
		     3 - main, master, slave, clientlink, mastermpi
*/

#define DEBUG 3

int main(int argc, char **argv)
{
	int my_rank, num_p;

	// fireup MPI and find out who I am
	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);
	MPI_Comm_size(MPI_COMM_WORLD, &num_p);

	// really high tech way to delete the file, MPE_IO_Stdout
	// doesn't seem to do this
	ofstream fout;
	char name[8];
	sprintf(name, "%d", my_rank);
	fout.open(name);
	fout.close();

	MPE_IO_Stdout_to_file("%d", 0644);
	
	if (my_rank==0)
	{
		/***************** master code *******************/

		// find out if socket port number were passed on the command line
		
		int port = 5000;
		int dataport = 5001;

		if (argc>1)
		{
			port = (int) atol(argv[1]);
                        dataport = port+1;
		}

		if (DEBUG)
			printf("Master being create on port=%d dataport=%d\n",
					port, dataport);

		Master master(num_p, port, dataport, DEBUG);

		/* main loop */

		// wait for somebody to connect
		// then init our parameters, shapes, etc

		master.loop();
	}
	else
	{
		/***************** slave code ********************/

		if (DEBUG)
			printf("Slave starting up, my_rank=%d\n",
					my_rank);

		// calculate who will be above and below us on the grid
		int north_rank = -1;
		int south_rank = -1;

		if (my_rank != 1)
			south_rank = my_rank - 1;
		if (my_rank != num_p-1)
			north_rank = my_rank + 1;

		Slave slave(my_rank, north_rank, south_rank, DEBUG);

		slave.loop();
	}

	MPI_Barrier(MPI_COMM_WORLD);

	MPI_Finalize();
}

